
from textual.app import ComposeResult
from textual.widgets import Label, DataTable
from textual.containers import VerticalGroup, VerticalScroll

USER_COLUMNS = [
    ("User", "Owner", "dfltgrp", "SOA", "RIRP", "UID", "Shell", "Home", "Last logon", "Created"),
]

DATASET_COLUMNS = [
    ("Dataset", "UACC", "Owner", "Created"),
]

class PanelResultsMixedType(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield DataTable(id="results_user_table")
        yield DataTable(id="results_dataset_table")
        yield DataTable(id="results_resource_table")

class PanelResultsSingleType(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield DataTable()

class PanelResultUser(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield DataTable()        

class PanelResultDataset(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield DataTable()        


from textual.theme import Theme

cynosure_theme = Theme(
    name="cynosure",
    primary="#e8e1c1",
    secondary="#ff0000",
    accent="#ff0000",
    foreground="#beb986",
    background="#112835",
    success="#beb986",
    warning="#EBCB8B",
    error="#ff0000",
    surface="#3B4252",
    panel="#434C5E",
    dark=True,
    variables={
        "block-cursor-text-style": "none",
        "footer-key-foreground": "#88C0D0",
        "input-selection-background": "#81a1c1 35%",
    },
)


#User API module for Blackwall Protocol, this wraps RACFU to increase ease of use and prevent updates from borking everything

from dataclasses import dataclass, field
from .traits_base import TraitsBase

#Checks if RACFU can be imported
try:
    from racfu import racfu # type: ignore
    racfu_enabled = True
except:
    print("##BLKWL_ERROR_2 Warning: could not find RACFU, entering lockdown mode")    
    racfu_enabled = False

@dataclass
class BaseUserTraits(TraitsBase):
    #primary
    owner: str
    default_group: str
    name: str | None = None
    installation_data: str | None = None

    password: str | None = field(default=None, metadata={
        "masked": True,
        "maximum": 8,
    })
    
    passphrase: str | None = field(default=None, metadata={
        "masked": True,
        "minimum": 12,
    })
    #user attributes
    special: bool | None = None
    operations: bool | None = None
    auditor: bool | None = None
    
    default_group_authority: str | None = None
    security_category: str | None = None
    security_categories: str | None = None
    class_authorization: str | None = None

@dataclass
class CICSUserTraits(TraitsBase):
    operator_class: str | None = None
    operator_id: str | None = None
    operator_priority: str | None = None
    resource_security_level_key: str | None = None
    resource_security_level_keys: str | None = None
    timeout: str | None = None
    transaction_security_level_key: str | None = None
    force_signoff_when_xrf_takeover: bool | None = None

@dataclass
class DCEUserTraits(TraitsBase):
    auto_login: bool | None = None
    name: str | None = None
    home_cell: str | None = None
    home_cell_uuid: str | None = None
    uuid: str | None = None

@dataclass
class DFPUserTraits(TraitsBase):
    data_application: str | None = None
    data_class: str | None = None
    management_class: str | None = None
    storage_class: str | None = None

@dataclass
class EIMUserTraits(TraitsBase):
    ldap_bind_profile: str | None = None

@dataclass
class KerbUserTraits(TraitsBase):
    encryption_algorithm: str | None = None
    name: str | None = None
    key_from: str | None = None
    key_version: str | None = None
    max_ticket_life: int | None = None

@dataclass
class LanguageUserTraits(TraitsBase):
    primary: str | None = None
    secondary: str | None = None

@dataclass
class LnotesUserTraits(TraitsBase):
    zos_short_name: str | None = None

@dataclass
class MfaUserTraits(TraitsBase):
    factor: str | None = None
    active: bool | None = None
    tags: str | None = None
    password_fallback: bool | None = None
    mfa_policy: str | None = None

@dataclass
class NDSUserTraits(TraitsBase):
    username: str | None = None

@dataclass
class NetviewUserTraits(TraitsBase):
    default_mcs_console_name: str | None = None
    security_control_check: str | None = None
    domain: str | None = None
    logon_commands: str | None = None
    receive_unsolicited_messages: str | None = None
    operator_graphic_monitor_facility_administration_allowed: str | None = None
    operator_graphic_monitor_facility_display_authority: str | None = None
    operator_scope_classes: str | None = None

@dataclass
class OMVSUserTraits(TraitsBase):
    uid: int | None = None
    home_directory: str | None = None
    auto_uid: bool | None = None
    max_address_space_size: int | None = None
    max_cpu_time: int | None = None
    max_files_per_process: int | None = None
    home_directory: str | None = None
    max_non_shared_memory: str | None = None
    max_file_mapping_pages: int | None = None
    max_processes: int | None = None
    default_shell: str | None = None
    shared: bool | None = None
    max_shared_memory: int | None = None
    max_threads: int | None = None

@dataclass
class OperparmUserTraits(TraitsBase):
    alternate_console_group: str | None = None
    receive_automated_messages: str | None = None
    command_target_system: str | None = None
    receive_delete_operator_messages: str | None = None
    receive_hardcopy_messages: str | None = None
    receive_internal_console_messages: str | None = None
    console_searching_key: str | None = None
    message_level: str | None = None
    log_command_responses: str | None = None
    message_format: str | None = None
    migration_id: str | None = None
    monitor_event: str | None = None
    message_scope: str | None = None
    console_authority: str | None = None
    receive_routing_code: str | None = None
    message_queue_storage: str | None = None
    receive_undelivered_messages: str | None = None
    receive_unknown_console_id_messages: str | None = None

@dataclass
class OvmUserTraits(TraitsBase):
    file_system_root: str | None = None
    home_directory: str | None = None
    default_shell: str | None = None
    uid: str | None = None

@dataclass
class ProxyUserTraits(TraitsBase):
    bind_distinguished_name: str | None = None
    bind_password: str | None = None
    ldap_host: str | None = None

@dataclass
class TSOUserTraits(TraitsBase):
    account_number: str | None = None
    logon_command: str | None = None
    sysout_destination_id: str | None = None
    hold_class: str | None = None
    job_class: str | None = None
    max_region_size: int | None = None
    message_class: str | None = None
    logon_procedure: str | None = None
    security_label: str | None = None
    default_region_size: int | None = None
    sysout_class: str | None = None
    data_set_allocation_unit: str | None = None
    user_data: str | None = None

@dataclass
class WorkattrUserTraits(TraitsBase):
    account_number: str | None = None
    sysout_building: str | None = None
    sysout_department: str | None = None
    sysout_user: str | None = None
    sysout_room: str | None = None
    sysout_email: str | None = None

if racfu_enabled:
    #User functions
    def user_exists(username: str):
        """Checks if a user exists, returns true or false"""
        result = racfu({"operation": "extract", "admin_type": "user", "profile_name": username})
        return result.result["return_codes"]["racf_return_code"] == 0
        
    def user_get(username: str):
        """Doesn't handle users that don't exist, recommend using user_exists() first"""
        result = racfu({"operation": "extract", "admin_type": "user", "profile_name": username})
        return result.result

    def user_create(
            username: str, 
            base: BaseUserTraits, 
            cics: CICSUserTraits | None = None, 
            dce: DCEUserTraits | None = None, 
            dfp: DFPUserTraits | None = None, 
            eim: EIMUserTraits | None = None, 
            language: LanguageUserTraits | None = None, 
            lnotes: LnotesUserTraits | None = None, 
            mfa: MfaUserTraits | None = None, 
            nds: NDSUserTraits | None = None, 
            netview: NetviewUserTraits | None = None, 
            omvs: OMVSUserTraits | None = None,
            operparm: OperparmUserTraits | None = None, 
            ovm: OvmUserTraits | None = None, 
            proxy: ProxyUserTraits | None = None, 
            tso: TSOUserTraits | None = None, 
            workattr: WorkattrUserTraits | None = None, 
            ):
        """Creates a new user, returns true if the user was successfully created and false if an error code was given"""
        traits = base.to_traits(prefix="base")
        
        if cics is not None:
            traits.update(cics.to_traits("cics"))

        if dce is not None:
            traits.update(dce.to_traits("dce"))

        if dfp is not None:
            traits.update(dfp.to_traits("dfp"))

        if eim is not None:
            traits.update(eim.to_traits("eim"))

        if language is not None:
            traits.update(language.to_traits("language"))

        if lnotes is not None:
            traits.update(lnotes.to_traits("lnotes"))

        if mfa is not None:
            traits.update(mfa.to_traits("mfa"))

        if nds is not None:
            traits.update(nds.to_traits("nds"))

        if netview is not None:
            traits.update(netview.to_traits("netview"))

        if omvs is not None:
            traits.update(omvs.to_traits("omvs"))

        if operparm is not None:
            traits.update(operparm.to_traits("operparm"))

        if ovm is not None:
            traits.update(ovm.to_traits("ovm"))

        if proxy is not None:
            traits.update(proxy.to_traits("proxy"))

        if tso is not None:
            traits.update(tso.to_traits("tso"))
        
        if workattr is not None:
            traits.update(workattr.to_traits("workattr"))

        result = racfu(
                {
                    "operation": "add", 
                    "admin_type": "user", 
                    "profile_name": username,
                    "traits":  traits
                }
            )
        return result.result["return_codes"]["racf_return_code"]

    def user_delete(username: str):
        result = racfu(
                {
                    "operation": "delete", 
                    "admin_type": "user", 
                    "profile_name": username,
                }
            )
        return result.result["return_codes"]["racf_return_code"] == 0

    def user_update(username: str):
        pass



from textual.app import ComposeResult
from textual.widgets import Button, Input, Label, RadioButton, RadioSet, TabbedContent, TabPane
from textual.containers import HorizontalGroup, VerticalScroll

from blackwall.panels.search.results import PanelResultsMixedType

class SearchSelector(HorizontalGroup):
    def compose(self) -> ComposeResult:
        with RadioSet(id="type-selector",classes="search-selector"):
            yield RadioButton("Any",value=True)
            yield RadioButton("User")
            yield RadioButton("Dataset profile")
            yield RadioButton("Resource profile")
        with RadioSet(id="filter-selector",classes="search-selector"):
            yield RadioButton("All",disabled=True)
            yield RadioButton("Only one",value=True)

class SearchField(HorizontalGroup):
    def action_search(self) -> None:
        # This is really stupid but it works *insert shrug emoji*
        tabs = self.parent.parent.parent.parent.parent.query_one(TabbedContent)
        tabs.add_pane(TabPane("Results",PanelResultsMixedType()))

    def compose(self) -> ComposeResult:
        yield Label("Search:")
        yield Input(name="Search",classes="search-field")
        yield Button("Search",action="search")

class PanelSearch(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield SearchSelector()
        yield SearchField()



class Positional:
    def __init__(self, positional: str):
        self.positional = positional

class Command:
    def __init__(self,command: str, short: str, alt: str, positionals: list[Positional]):
        self.command = command
        self.short = short
        self.alt = alt
        self.positionals = positionals

test_command = Command(command="TEST", short="T", alt="TE", positionals=[
    Positional("EMSE")
])


from abc import ABC
from dataclasses import dataclass, fields
from typing import Any

@dataclass
class TraitsBase(ABC):
    def to_traits(self, prefix: str) -> dict[str, Any]:
        traits = {}

        for field in fields(type(self)):
            value = getattr(self, field.name)
            field.metadata.get("masked", False)
            if value is not None:
                traits[f"{prefix}:{field.name}"] = value

        return traits



from textual.app import App, SystemCommand
from textual.widgets import Header, Footer, ListView, Label
from textual.containers import Container
from textual.screen import Screen

from typing import Iterable

try:
    from zoautil_py import zsystem # type: ignore
    zoau_enabled = True
except:
    print("##BLKWL_ERROR_1 Warning: could not find ZOAU, certain features will be disabled such as diplaying system and LPAR names")    
    zoau_enabled = False

import json
from .subcommands import Subcommands
from .command_line import TSOCommandField
from .command_line import CommandHistoryScreen
from .theme import cynosure_theme

from .tabs import TabSystem

#system information
if zoau_enabled:
    zsystem_info = json.loads(zsystem.zinfo()) # type: ignore
    system_name = zsystem_info["sys_info"]["sys_name"]
    lpar_name = zsystem_info["sys_info"]["lpar_name"]

class Blackwall(App):
    #Import css
    CSS_PATH = "UI.css"

    BINDINGS = [
        ("h", "push_screen('history')", "Switch to command history view")
    ]

    #This portion handles the text in the header bar
    def on_mount(self) -> None:
        self.title = "Blackwall Protocol"
        self.sub_title = "Mainframe Security Administration"
        self.register_theme(cynosure_theme)
        self.theme = "cynosure"
        self.install_screen(CommandHistoryScreen(), name="history")

    def get_system_commands(self, screen: Screen) -> Iterable[SystemCommand]:
        yield from super().get_system_commands(screen)  
        yield Subcommands()

    #UI elements
    def compose(self):
        #display system and LPAR name
        with Container():
            if zoau_enabled:
                yield Label(f"You are working on the {system_name} mainframe system in LPAR {lpar_name}")
            yield TSOCommandField()
            yield Header()
            yield TabSystem()
            yield ListView()
            yield Footer()




from blackwall.api.user import user_exists, user_get
from blackwall.api.dataset import dataset_profile_exists, dataset_profile_get
from blackwall.api.resource import resource_profile_exists, resource_profile_get

def user_possible(query: str):
    return len(query) <= 8 and query.isalnum()

def search_user(query: str):
    if user_possible(query):
        if user_exists(query):
            return user_get(query)

def search_dataset(query: str):
    if dataset_profile_exists(query):
        return dataset_profile_get(query)

def search_resource(query: str):
    if resource_profile_exists(query):
        return resource_profile_get(query)

def search_database_query_one(query: str, query_type: str):
    if query_type == "any":
        user_result = search_user(query)
        dataset_result = search_dataset(query)
        resource_result = search_resource(query)
    elif query_type == "user":
        user_result = user_result = search_user(query)
    elif query_type == "dataset":
        dataset_result = search_dataset(query)
    elif query_type == "resource":
        resource_result = search_resource(query)

def search_database_query_multiple(query: str, query_type: str):
    if query_type == "any":
        pass




from textual.app import ComposeResult
from textual.widgets import Button, Label, RadioButton, RadioSet, Log
from textual.containers import HorizontalGroup, VerticalGroup, VerticalScroll, Horizontal

try:
    from zoautil_py import zsystem # type: ignore
    zoau_enabled = True
except:
    print("##BLKWL_ERROR_1 Warning: could not find ZOAU, disabling APF health checks")    
    zoau_enabled = False

class AnalysisSelector(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Select desired health checks: ")
        with RadioSet(id="selector-health-checks",classes="selector-health-checks"):
            yield RadioButton("UACC best practice",value=True, tooltip="Checks if UACC is being used instead of any=*")
            yield RadioButton("z/OS Unix excessive permissions",tooltip="Checks if files have RWX set for everyone on the system")
            yield RadioButton("Unprotected APF datasets",tooltip="Checks if APF datasets are unprotected by a dataset profile")
            yield RadioButton("Unused users",tooltip="Checks if users haven't been used in a really long time")

class AnalysisLog(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Log()

class AnalysisConfirm(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Button("Run",classes="analysis-confirm")

class AnalysisMainView(HorizontalGroup):
    def compose(self) -> ComposeResult:
        yield AnalysisSelector()
        yield AnalysisLog()

class PanelAnalysis(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield AnalysisMainView()
        yield AnalysisConfirm()


#General resource API module for Blackwall Protocol, this wraps RACFU to increase ease of use and prevent updates from borking everything

from dataclasses import dataclass, fields
from .traits_base import TraitsBase

#Checks if RACFU can be imported
try:
    from racfu import racfu # type: ignore
    racfu_enabled = True
except:
    print("##BLKWL_ERROR_2 Warning: could not find RACFU, entering lockdown mode")    
    racfu_enabled = False

if racfu_enabled:
    #General resource profile function
    def resource_profile_exists(resource: str):
        """Checks if a general resource profile exists, returns true or false"""
        result = racfu({"operation": "extract", "admin_type": "resource", "profile_name": resource})
        return result.result["return_codes"]["racf_return_code"] == "0"

    def resource_profile_get(resource: str):
        """Doesn't handle general resource profiles that don't exist, recommend using resource_profile_exists() first"""
        result = racfu({"operation": "extract", "admin_type": "resource", "profile_name": resource})
        return result.result

    def resource_profile_create():
        pass

    def resource_profile_delete(resource: str):
        pass

    def resource_profile_update(resource: str):
        pass


from . import commands_definition as defintion
from textual.app import ComposeResult

from datetime import datetime

from textual import on
from textual.events import ScreenResume
from textual.widgets import Input, Log, Label
from textual.containers import HorizontalGroup
from textual.screen import Screen

import subprocess

command_history = ""



def generate_command_meta_header(command):
    now = datetime.now() # current date and time
    date_time = now.strftime("d-%m-%d-%Y-t-%H-%M-%S")
    return f"""
    --------------------------------------------------------------------------------------------------
    Command '{command}' 
    executed on {date_time}
    --------------------------------------------------------------------------------------------------
    \n
    """

class CommandHistoryScreen(Screen):
    BINDINGS = [("escape", "app.pop_screen", "Pop screen")]

    
    def compose(self) -> ComposeResult:
        yield Label("Command history: ")
        yield Log()

    @on(ScreenResume)
    def on_resume(self):
        log = self.query_one(Log)
        log.clear()
        log.write(command_history)

class TSOCommandField(HorizontalGroup):
    def compose(self) -> ComposeResult:
        yield Input(id="cli",max_length=250,classes="commands",tooltip="Use this command field to submit TSO and RACF commands. You can view the output in the command history panel")

    @on(Input.Submitted)
    def execute_command(self) -> None:
        global command_history
        command = self.query_exactly_one(selector="#cli").value
        if command != "":
            try:
                output = subprocess.run(f'tsocmd "{command}"' , shell=True, check=True, capture_output=True,encoding="utf-8")
                command_history = command_history + generate_command_meta_header(command) + output.stdout
                self.notify(f"command {command.upper()} successfully completed",severity="information")
            except BaseException as e:
                self.notify(f"Command {command.upper()} failed: {e}",severity="error")



from textual.app import ComposeResult
from textual.widgets import Button, TabPane, TabbedContent
from textual.containers import HorizontalGroup

from .panels.users.user import PanelUser
from .panels.search.search import PanelSearch
from .panels.analysis.analysis import PanelAnalysis

class TabSystem(HorizontalGroup):
    BINDINGS = [
        ("u", "open_user_administration", "Open user tab"),
        ("f", "open_search", "Open search tab"),
        ("l", "open_analysis", "Open analysis tab"),
        ("r", "remove", "Remove active tab"),
        ("c", "clear", "Clear all tabs"),
    ]

    def compose(self) -> ComposeResult:
        yield TabbedContent()

    #Add new tab
    def action_open_user_administration(self) -> None:
        """Add a new user administration tab."""
        tabs = self.query_one(TabbedContent)
        tabs.add_pane(TabPane("User management",PanelUser()))

    def action_open_search(self) -> None:
        """Add a new search tab."""
        tabs = self.query_one(TabbedContent)
        tabs.add_pane(TabPane("Search",PanelSearch()))

    def action_open_analysis(self) -> None:
        """Add a new analysis tab."""
        tabs = self.query_one(TabbedContent)
        tabs.add_pane(TabPane("Health check",PanelAnalysis()))

    #Remove current tab
    def action_remove(self) -> None:
        """Remove active tab."""
        tabs = self.query_one(TabbedContent)
        active_pane = tabs.active_pane
        if active_pane is not None:
            tabs.remove_pane(active_pane.id)

    #Clear all tabs
    def action_clear(self) -> None:
        """Clear the tabs."""
        self.query_one(TabbedContent).clear_panes()


from typing import Iterable

from textual.app import SystemCommand
from textual.screen import Screen

class Subcommands():
    def get_system_commands(self) -> Iterable[SystemCommand]:
        yield SystemCommand("User creation", "Open a new tab with the user creation panel", self.bell)  


from textual.app import ComposeResult
from textual.widgets import Button, Label, Select, Input
from textual.containers import HorizontalGroup, VerticalGroup, VerticalScroll, Horizontal, Right

class PanelDatasetName(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Profile name")
        yield Input()

class PanelDatasetAudit(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Audit")
        yield Select([("ALL", 1),("FAILURES", 2),("NONE", 3),("SUCCESS", 4)],value=2,classes="uacc-select")

class PanelDatasetUACC(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("UACC")
        yield Select([("NONE", 1),("READ", 2),("EXECUTE", 3),("UPDATE", 4),("CONTROL", 5),("ALTER", 6)],value=1,classes="uacc-select")

class PanelDatasetNotify(VerticalGroup):
    def compose(self) -> ComposeResult:
        yield Label("Notify user")
        yield Input() 

class PanelDatasetAccessSettings(HorizontalGroup):
    def compose(self) -> ComposeResult:
        yield PanelDatasetUACC()
        yield PanelDatasetAudit()
        yield PanelDatasetNotify()

class PanelDatasetActionButtons(HorizontalGroup):
    def compose(self) -> ComposeResult:
        yield Button("Save",classes="action-button")
        yield Button("Delete",classes="action-button")

class PanelDataset(VerticalScroll):
    def compose(self) -> ComposeResult:
        yield PanelDatasetName()
        yield PanelDatasetAccessSettings()
        yield PanelDatasetActionButtons()


#API module for Blackwall Protocol, this wraps RACFU to increase ease of use and prevent updates from borking everything

from dataclasses import dataclass
from .traits_base import TraitsBase

#Checks if RACFU can be imported
try:
    from racfu import racfu # type: ignore
    racfu_enabled = True
except:
    print("##BLKWL_ERROR_2 Warning: could not find RACFU, entering lockdown mode")    
    racfu_enabled = False

@dataclass
class BaseDatasetTraits(TraitsBase):
    owner: str
    audit_alter: str | None = None
    audit_control: str | None = None
    audit_none: str | None = None
    audit_read: str | None = None
    audit_read: str | None = None
    audit_update: str | None = None
    security_category: str | None = None
    installation_data: str | None = None
    data_set_type: str | None = None
    erase_data_sets_on_delete: bool | None = None
    model_profile_class: str | None = None
    model_profile_generic: str | None = None
    tape_data_set_file_sequence_number: int | None = None
    model_profile: str | None = None
    model_profile_volume: str | None = None
    global_audit_alter: str | None = None
    global_audit_control: str | None = None
    global_audit_none: str | None = None
    global_audit_read: str | None = None
    global_audit_update: str | None = None
    level: int | None = None
    data_set_model_profile: str | None = None
    notify_userid: str | None = None
    auditing: str | None = None
    security_label: str | None = None
    security_level: str | None = None
    racf_indicated_dataset: str | None = None
    create_only_tape_vtoc_entry: str | None = None
    universal_access: str | None = None
    data_set_allocation_unit: str | None = None
    volume: str | None = None
    warn_on_insufficient_access: str | None = None

#Checks if RACFU can be imported
try:
    from racfu import racfu # type: ignore
    racfu_enabled = True
except:
    print("##BLKWL_ERROR_2 Warning: could not find RACFU, entering lockdown mode")    
    racfu_enabled = False

if racfu_enabled:
    #Dataset functions
    def dataset_profile_exists(dataset: str):
        """Checks if a dataset profile exists, returns true or false"""
        result = racfu({"operation": "extract", "admin_type": "data-set", "profile_name": dataset})
        return result.result["return_codes"]["racf_return_code"] == "0"

    def dataset_profile_get(dataset: str):
        """Doesn't handle dataset profiles that don't exist, recommend using dataset_profile_exists() first"""
        result = racfu({"operation": "extract", "admin_type": "data-set", "profile_name": dataset})
        return result.result

    def dataset_profile_create():
        pass

    def dataset_profile_delete(dataset: str):
        pass

    def dataset_profile_update(dataset: str):
        pass



#Project by Emma Skovgaard

#Enables true colors to be active by default
from os import environ
if "TEXTUAL_COLOR_SYSTEM" not in environ:
    environ["TEXTUAL_COLOR_SYSTEM"] = "truecolor"

from .app import Blackwall

def main():
    Blackwall().run()

if __name__ == "__main__":
    main()




from enum import Enum
from dataclasses import dataclass
from textual.reactive import reactive
from textual.app import ComposeResult
from textual.widgets import Input, Label, Button, RadioButton, Collapsible
from textual.containers import HorizontalGroup, VerticalGroup, VerticalScroll

from blackwall.api import user

class PanelMode(Enum):
    create = 1
    edit = 2
    read = 3

class PanelUserInfo(HorizontalGroup):
    edit_mode: reactive[PanelMode] = reactive(PanelMode.create,recompose=True)

    def compose(self) -> ComposeResult:
        if self.edit_mode != PanelMode.create:
            yield Label("Created: ")
            yield Label("Last logon: ")

class PanelUserName(HorizontalGroup):
    """Username and name components"""
    username: reactive[str] = reactive("")
    name: reactive[str] = reactive("")

    edit_mode: reactive[PanelMode] = reactive(PanelMode.create,recompose=True)

    if edit_mode == True:
        is_disabled = True
    else:
        is_disabled = False

    def compose(self) -> ComposeResult:
        yield Label("Username*: ")
        yield Input(max_length=8,id="username",classes="username",tooltip="Username is what the user uses to log on with, this is required. While very few characters can be used at least 4 character long usernames are recommended to avoid collisions",disabled=self.is_disabled).data_bind(value=PanelUserName.username)
        yield Label("name: ")
        yield Input(max_length=20,id="name",classes="name",tooltip="For personal users this is typically used for names i.e. Song So Mi, for system users it can be the name of the subsystem that it is used for").data_bind(value=PanelUserName.name)

class PanelUserOwnership(HorizontalGroup):
    """Component that contains ownership field and default group"""
    def compose(self) -> ComposeResult:
        yield Label("Owner*: ")
        yield Input(max_length=8,id="owner",classes="owner", tooltip="The group or user that owns this user profile. This is required in the RACF database")
        yield Label("Default group*: ")
        yield Input(max_length=8,id="default_group",classes="owner", tooltip="All users must belong to a group in the RACF database")

class PanelUserPassword(VerticalGroup):
    """Change/add password component"""
    def compose(self) -> ComposeResult:
        with Collapsible(title="Password"):
            yield Label("Passwords can only be 8 characters long")
            yield Label("New password:")
            yield Input(max_length=8,id="password",classes="password",password=True)
            yield Label("Repeat password*:")
            yield Input(max_length=8,id="password_repeat",classes="password",password=True)

class PanelUserPassphrase(VerticalGroup):
    """Change/add passphrase component"""
    def compose(self) -> ComposeResult:
        with Collapsible(title="Passphrase"):
            yield Label("Passphrases need to be between 12 and 100 characaters long")
            yield Label("New passphrase:")
            yield Input(max_length=100,id="passphrase",classes="passphrase",password=True)
            yield Label("Repeat passphrase*:")
            yield Input(max_length=100,id="passphrase_repeat",classes="passphrase",password=True)
    
class PanelUserAttributes(VerticalGroup):
    """User attributes component"""
    def compose(self) -> ComposeResult:
        with Collapsible(title="User attributes"):
            yield RadioButton("Special",id="user_attribute_special",tooltip="This is RACF's way of making a user admin. Special users can make other users special, making this a potentially dangerous option")
            yield RadioButton("Operations",id="user_attribute_operations",tooltip="This is a very dangerous attribute that allows you to bypass most security checks on the system, this should only be used during maintenance tasks and removed immediately afterwards")
            yield RadioButton("Auditor",id="user_attribute_auditor")
            yield RadioButton("Read only auditor (ROAUDIT)",id="user_attribute_roaudit")

class PanelUserSegments(VerticalGroup):
    """Component where the user can add segments such as the OMVS segment"""
    def compose(self) -> ComposeResult:
        with Collapsible(title="User segments"):
            with Collapsible(title="TSO"):
                yield RadioButton("TSO enabled",id="user_segment_tso")
            with Collapsible(title="OMVS"):
                yield RadioButton("OMVS",id="user_segment_omvs")
                yield Label("UID: ")
                yield Input(max_length=30,id="uid",classes="username",type="integer")
                yield Label("Home directory: ")
                yield Input(max_length=255,id="home_directory",classes="username")
                yield Label("Shell path: ")
                yield Input(max_length=255,id="shell",classes="username")
            with Collapsible(title="CSDATA"):    
                yield RadioButton("CSDATA",id="user_segment_csdata")
            with Collapsible(title="KERB"):   
                yield RadioButton("KERB",id="user_segment_kerb")
            with Collapsible(title="LANGUAGE"):   
                yield RadioButton("LANGUAGE",id="user_segment_language")
            with Collapsible(title="OPERPARM"):   
                yield RadioButton("OPERPARM",id="user_segment_operparm")
            with Collapsible(title="OVM"):   
                yield RadioButton("OVM",id="user_segment_ovm")
            with Collapsible(title="NDS"): 
                yield RadioButton("NDS",id="user_segment_nds")
            with Collapsible(title="DCE"): 
                yield RadioButton("DCE",id="user_segment_dce")
            with Collapsible(title="DFP"): 
                yield RadioButton("DFP",id="user_segment_dfp")
            with Collapsible(title="CICS"): 
                yield RadioButton("CICS",id="user_segment_cics")

class PanelUserActionButtons(HorizontalGroup):
    """Action buttons"""
    edit_mode: reactive[PanelMode] = reactive(PanelMode.create,recompose=True)

    if edit_mode == True:
        is_disabled = True
    else:
        is_disabled = False

    def __init__(self, save_action: str, delete_action: str):
        super().__init__()
        self.save_action = save_action
        self.delete_action = delete_action

    def compose(self) -> ComposeResult:
        if self.edit_mode == PanelMode.create:
            yield Button("Create", tooltip="This will update the user, or create it if the user doesn't exist",action="save",classes="action-button",id="save")
        elif self.edit_mode == PanelMode.edit:
            yield Button("Save", tooltip="This will update the user, or create it if the user doesn't exist",action="save",classes="action-button",id="save")
        yield Button("Delete", tooltip="This will delete the user permanently from the RACF database",id="delete",action="delete",classes="action-button",disabled=self.is_disabled)

    async def action_save(self):
        await self.app.run_action(self.save_action,default_namespace=self.parent)

    async def action_delete(self):
        await self.app.run_action(self.delete_action,default_namespace=self.parent)

@dataclass
class UserInfo:
    mode: PanelMode = PanelMode.create
    username: str = ""
    name: str = ""
    owner: str = ""
    dfltgrp: str = ""
    installation_data: str = ""

class PanelUser(VerticalScroll):
    user_info: reactive[UserInfo] = reactive(UserInfo)

    def compose(self) -> ComposeResult:
        yield PanelUserInfo()
        yield PanelUserName()
        yield PanelUserOwnership()
        yield PanelUserPassword()
        yield PanelUserPassphrase()
        yield PanelUserAttributes()
        yield PanelUserSegments()
        yield PanelUserActionButtons(save_action="save_user", delete_action="delete_user")
    
    def watch_user_info(self, value: UserInfo):
        user_name_panel = self.query_exactly_one(PanelUserName)
        #valid modes: create, edit, and read
        user_name_panel.mode = value.mode
        user_name_panel.username = value.username
        user_name_panel.name = value.name
        user_name_panel.owner = value.owner
        user_name_panel.dfltgrp = value.dfltgrp
        user_name_panel.installation_data = value.installation_data

    def set_edit_mode(self):
        user_name_panel = self.query_exactly_one(PanelUserName)
        user_name_panel.mode = PanelMode.edit
        self.query_exactly_one(selector="#username").disabled = True
        self.query_exactly_one(selector="#delete").disabled = False
        self.query_exactly_one(selector="#save").label = "Save"
        self.notify(f"Switched to edit mode",severity="information")

    def action_delete_user(self) -> None:
        pass

    def action_save_user(self) -> None:
        username = self.query_exactly_one(selector="#username").value
        name = self.query_exactly_one(selector="#name").value
        owner = self.query_exactly_one(selector="#owner").value
        default_group = self.query_exactly_one(selector="#default_group").value
        password = self.query_exactly_one(selector="#password").value
        passphrase = self.query_exactly_one(selector="#passphrase").value

        special = self.query_exactly_one(selector="#user_attribute_special").value
        operations = self.query_exactly_one(selector="#user_attribute_operations").value
        auditor = self.query_exactly_one(selector="#user_attribute_auditor").value
        if not user.user_exists(username=username):
            result = user.user_create(
                username=username,
                base=user.BaseUserTraits(
                    owner=owner,
                    name=name,
                    default_group=default_group,
                    password=password,
                    passphrase=passphrase,
                    special=special,
                    operations=operations,
                    auditor=auditor
                                        )
                )
            if (result == 0 or result == 4):
                self.notify(f"User {username} created, return code: {result}",severity="information")
                self.set_edit_mode()
            else:
                self.notify(f"Unable to create user, return code: {result}",severity="error")
        else:
            pass



#Group API module for Blackwall Protocol, this wraps RACFU to increase ease of use and prevent updates from borking everything

from dataclasses import dataclass
from .traits_base import TraitsBase

#Checks if RACFU can be imported
try:
    from racfu import racfu # type: ignore
    racfu_enabled = True
except:
    print("##BLKWL_ERROR_2 Warning: could not find RACFU, entering lockdown mode")    
    racfu_enabled = False

@dataclass
class BaseGroupTraits(TraitsBase):
    owner: str
    installation_data: str | None = None
    superior_group: str
    terminal_universal_access: str | None = None
    universal: str | None = None

@dataclass
class DFPGroupTraits(TraitsBase):
    data_application: str | None = None
    data_class: str | None = None
    management_class: str | None = None
    storage_class: str | None = None

@dataclass
class OMVSGroupTraits(TraitsBase):
    auto_gid: str | None = None
    gid: str | None = None
    shared: str | None = None

@dataclass
class OVMGroupTraits(TraitsBase):
    gid: str | None = None
    home_directory: str | None = None

@dataclass
class TMEGroupTraits(TraitsBase):
    roles: str | None = None

if racfu_enabled:
    #Group functions
    def group_profile_exists(group: str):
        """Checks if a group exists, returns true or false"""
        result = racfu({"operation": "extract", "admin_type": "group", "profile_name": group})
        return result.result["return_codes"]["racf_return_code"] == "0"
    
    def group_get(group: str):
        pass

    def group_get_connections(group: str):
        """Get information on group connections"""
        pass

    def group_create():
        pass

    def group_delete(group: str):
        pass

    def group_update(group: str):
        pass